;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  LITECODE.ASM                         R.Trevithick, Sun  04-16-1989  ;;
;;                                                                      ;;
;;  Password number indicated in binary by keyboard status lights, set  ;;
;;  to a relatively random state by sampling the system time.           ;;
;;                                                                      ;;
;;             CAPS   NUM   SCROLL          Number to enter             ;;
;;             ----   ---   ------          ---------------             ;;
;;              0      0      0        =           0                    ;;
;;              0      0      1        =           1                    ;;
;;              0      1      0        =           2                    ;;
;;              0      1      1        =           3                    ;;
;;              1      0      0        =           4                    ;;
;;                                                                      ;;
;;              and so on...                                            ;;
;;                                                                      ;;
;;  Somewhere in the entered password must be the number indicated by   ;;
;;  the keyboard lights, and this must be the _first_ number entered.   ;;
;;  You can enter other characters to confuse onlookers, and terminate  ;;
;;  the password with the <ENTER> key.  Keyboard locks are zeroed, and  ;;
;;  first number entered is compared to correct value.  Program resets  ;;
;;  on each error -- displaying a new light configuration to use.       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BIOS_SEG        EQU     0040h                   ; BIOS data segment
TIME_VAL        EQU     byte ptr ds:[006ch]     ; seconds count offset
KEY_STAT        EQU     byte ptr ds:[0017h]     ; keyboard status offset
KEY_MASK        EQU     01110000b               ; Caps, Num and Scroll mask


       .MODEL   TINY
       .CODE

        ORG     0100h

START:  mov     ah, 0fh                         ; clear the screen
        int     10h
        xor     ah, ah
        int     10h

        mov     ax, BIOS_SEG                    ; set data segment
        mov     ds, ax


;-------------------------------------------------------------------------;
; This is the re-entry point on each password error - reset the password. ;
;-------------------------------------------------------------------------;
MAIN:   mov     bl, TIME_VAL                    ; pseudo rand()
        and     bl, KEY_MASK                    ; mask Caps, Num, Scroll
        mov     KEY_STAT, bl                    ; set locks (and lights)

        mov     cl, 4                           ; get ASCII value of the
        shr     bl, cl                          ;  3 lock bits, which is
        add     bl, '0'                         ;  the 'password' char.

        xor     si, si                          ; initialize 'correct' flag
        xor     di, di                          ; ..and 'got a number' flag


;-------------------------------------------------------------------------;
; Now we loop until a CR is hit, absorbing each character unless it's the ;
; first numeric key the user has hit -- in which case we test it and set  ;
; the flags.  Keeping shift keys cleared prevents Ctrl-Alt-Del warm boot. ;
;-------------------------------------------------------------------------;
KEYIN:  and     KEY_STAT, KEY_MASK              ; clear all shift keys
        mov     ah, 1                           ; see if key available
        int     16h
        jz      KEYIN                           ; no, keep waiting

        mov     ah, 07h                         ; yes, read it into al
        int     21h

        cmp     al, 0dh                         ; finished?
        jz      CHECK                           ; yes, go check flags...

        or      di, di                          ; got a number yet?
        jnz     KEYIN                           ; yes, skip rest of tests

        cmp     al, '0'                         ; see if it's a number key
        jb      KEYIN
        cmp     al, '9'
        ja      KEYIN

        inc     di                              ; set 'got a number' flag
        cmp     al, bl                          ; see if correct
        jne     KEYIN                           ; no, they've bombed
        inc     si                              ; yes, set 'correct' flag
        jmp     short KEYIN                     ; and keep waiting for CR


;------------------------------------------------------------------------;
; All that's left is to clear key locks and see if they 'passed' or not. ;
;------------------------------------------------------------------------;
CHECK:  mov     KEY_STAT, 0                     ; zero all key locks
        or      si, si                          ; check 'correct' flag
        jz      MAIN                            ; bogus, reset system!

        ret                                     ; ok, got it right.

        END     START

